import sys

from libcloud.compute.drivers.ec2 import EC2NodeDriver as LibCloudEC2NodeDriver
from libcloud.compute.drivers.ec2 import NAMESPACE
from libcloud.utils.xml import fixxpath, findtext


class EC2VPNGateway(object):
    """
    Class which stores information about VPN Gateways.

    Note: This class is VPC specific.
    """

    def __init__(self, id, name, state, extra=None):
        self.id = id
        self.name = name
        self.state = state
        self.extra = extra or {}

    def __repr__(self):
        return '<VPNGateway: id=%s>' % self.id


class EC2NodeDriver(LibCloudEC2NodeDriver):

    def ex_create_vpn_gateway(self, name=None):

        params = {'Action': 'CreateVpnGateway', 'Type': 'ipsec.1'}

        resp = self.connection.request(self.path, params=params).object

        element = resp.findall(fixxpath(xpath='vpnGateway',
                                        namespace=NAMESPACE))

        gateway = self._to_vpn_gateway(element[0], name)

        if name and self.ex_create_tags(gateway, {'Name': name}):
            gateway.extra['tags']['Name'] = name

        return gateway

    def _to_vpn_gateways(self, object, xpath):
        return [self._to_vpn_gateway(el)
                for el in object.findall(fixxpath(xpath=xpath,
                                                  namespace=NAMESPACE))]

    def _to_vpn_gateway(self, element, name=None):
        id = findtext(element=element,
                      xpath='vpnGatewayId',
                      namespace=NAMESPACE)

        state = findtext(element=element,
                         xpath='attachmentSet/item/state',
                         namespace=NAMESPACE)

        # If there's no attachment state, let's
        # set it to available
        if not state:
            state = 'available'

        # Get our tags
        tags = self._get_resource_tags(element)

        # If name was not passed into the method then
        # fall back then use the gateway id
        name = name if name else tags.get('Name', id)

        return EC2VPNGateway(id=id, name=name,
                             state=state,
                             extra={'tags': tags})

    def ex_attach_vpn_gateway(self, vpn_gateway, network):
        params = {'Action': 'AttachVpnGateway',
                  'VpnGatewayId': vpn_gateway.id,
                  'VpcId': network.id}

        res = self.connection.request(self.path, params=params).object

        return self._get_boolean(res)

    def ex_network_enable_dns_hostnames(self, network):
        params = {
            'Action': 'ModifyVpcAttribute',
            'VpcId': network.id,
            'EnableDnsHostnames.Value': True}
        res = self.connection.request(self.path, params=params).object
        return self._get_boolean(res)

    def ex_authorize_security_group_by_id(self, group_id, from_port, to_port, cidr_ip, protocol='tcp'):
        params = {'Action': 'AuthorizeSecurityGroupIngress',
                  'GroupId': group_id,
                  'IpProtocol': protocol,
                  'FromPort': str(from_port),
                  'ToPort': str(to_port),
                  'CidrIp': cidr_ip}
        try:
            res = self.connection.request(
                self.path, params=params.copy()).object
            return self._get_boolean(res)
        except Exception:
            e = sys.exc_info()[1]
            if e.args[0].find('InvalidPermission.Duplicate') == -1:
                raise e



# Clases base


class SecurityGroup:

    def __init__(self, id, name, network):
        self.id = id
        self.name = name
        self.network = network

    def allow_inbound_traffic(self, port):
        raise NotImplementedError('allow_inbound_traffic not implemented for this driver')

    def delete(self):
        raise NotImplementedError('delete not implemented for this driver')



from bastion.networking.route_table.base import RouteTable


class AzureRouteTable(RouteTable):

    def get_cloud_driver(self):
        return self.network.get_cloud_driver()

    def list_routes(self):
            raise NotImplementedError('list_routes not implemented for this driver')

    def create_route(self, cidr, internet_gateway=None):
            raise NotImplementedError('create_route not implemented for this driver')

    def list_subnet_associations(self):
            raise NotImplementedError('list_subnet_associations not implemented for this driver')

    def create_subnet_association(self, subnet):
            raise NotImplementedError('create_subnet_associations not implemented for this driver')

    def delete(self):
            raise NotImplementedError('delete not implemented for this driver')



# Security Group para Amazon


from bastion.networking.security_group.base import SecurityGroup
from bastion.component import Component


class AmazonSecurityGroup (SecurityGroup, Component):

    def get_cloud_driver(self):
        return self.network.get_cloud_driver()

    def allow_inbound_traffic(self, port):
        raise NotImplementedError('allow_inbound_traffic not implemented for this driver')

    def delete(self):
        raise NotImplementedError('delete not implemented for this driver')



import json
import os

from bastion.networking.private_dns.base import PrivateDns
from bastion.component import Component


class AzurePrivateDns(PrivateDns, Component):

    hostnames = []

    def __init__(self, dns_server, network):
        super(AzurePrivateDns, self).__init__(dns_server, network)
        hostname_element = {
            'name': 'ns1',
            'ip': dns_server.private_ips[0]
        }
        self.hostnames.append(hostname_element)

    def get_cloud_driver(self):
        return self.network.get_cloud_driver()

    def add_hostname(self, hostname, vm):

        hostname_element = {
            'name': hostname,
            'ip': vm.private_ips[0]
        }

        self.hostnames.append(hostname_element)

        dns_parameters = \
            {
                'dns_server_ip': self.dns_server.private_ips[0],
                'domain': self.domain,
                'hostnames': self.hostnames,
            }

        this_file_path = os.path.abspath(os.path.dirname(__file__))
        bind_playbook_path = os.path.join(this_file_path, "../../playbooks/bertvv-bind.yml")

        self.dns_server.provision(playbook_path=bind_playbook_path,
                                  parameters=json.dumps(dns_parameters))



# Clases base
import os

from bastion.provider_types import Providers


class BaseAuthDriver:
    _token = None
    _project_id = None

    def __init__(self, token, project_id):
        self._token = token
        self._project_id = project_id

    def get_provider_info(self, path):
        raise NotImplementedError(
            'get_providers_info not implemented for this driver')

    def get_providers_info(self, path):
        raise NotImplementedError(
            'get_providers_info not implemented for this driver')


class ProviderInfo:
    type = None
    cred = None
    prop = None

    def __init__(self):
        pass


class AWSProviderCred:

    access_key = None
    secret_key = None
    region = None
    public_key_path = None
    private_key_path = None

    def __init__(self, secret_aws):
        self.access_key = secret_aws['access_key']
        self.secret_key = secret_aws['secret_key']
        self.region = secret_aws['region']
        self.public_key_path = os.path.abspath(secret_aws['public_key_path'])
        self.private_key_path = os.path.abspath(secret_aws['private_key_path'])


class AWSProviderProp:

    image_id = None
    vpn_image_id = None
    size_id = None

    def __init__(self, secret_aws):
        self.image_id = secret_aws['image_id']
        self.vpn_image_id = secret_aws['vpn_image_id']
        self.size_id = secret_aws['size_id']


class AWSProviderInfo(ProviderInfo):

    def __init__(self, secret_aws):
        self.type = Providers.AWS
        self.cred = AWSProviderCred(secret_aws)
        self.prop = AWSProviderProp(secret_aws)


class AzureProviderCred:

    tenant_id = None
    subscription_id = None
    application_id = None
    password = None
    public_key = None
    private_key = None

    def __init__(self, secret_azure):
        self.tenant_id = secret_azure['tenant_id']
        self.subscription_id = secret_azure['subscription_id']
        self.application_id = secret_azure['application_id']
        self.password = secret_azure['password']
        self.public_key_path = secret_azure['public_key_path']
        self.private_key_path = secret_azure['private_key_path']


class AzureProviderProp:

    location_id = None
    resource_group = None
    storage_account = None
    image_id = None
    dns_image_id = None
    size_id = None

    def __init__(self, secret_azure):
        self.location_id = secret_azure['location_id']
        self.resource_group = secret_azure['resource_group']
        self.storage_account = secret_azure['storage_account']
        self.image_id = secret_azure['image_id']
        self.dns_image_id = secret_azure['dns_image_id']
        self.size_id = secret_azure['size_id']


class AzureProviderInfo(ProviderInfo):
    def __init__(self, secret_azure):
        self.type = Providers.AZURE
        self.cred = AzureProviderCred(secret_azure)
        self.prop = AzureProviderProp(secret_azure)



# Clases base
from bastion.networking.gateway.base import Gateway
from bastion.component import Component


class AWSGateway(Gateway, Component):

    def get_cloud_driver(self):
        return self.networking.get_cloud_driver()



# NetworkInterface para AWS

from bastion.networking.network_interface.base import NetworkInterface
from libcloud.compute.drivers.ec2 import EC2NetworkInterface
from bastion.component import Component


class AWSNetworkInterface (NetworkInterface, Component):

    def get_cloud_driver(self):
        return self.subnet.get_cloud_driver()

    def delete(self):
        cloud_driver = self.get_cloud_driver()
        cloud_nic = EC2NetworkInterface(id=self.id, name=None, state=None, extra=None)
        cloud_driver.ex_delete_network_interface(cloud_nic)



# Networking para Azure

from bastion.component import Component
from bastion.networking.base import Networking
from bastion.networking.network.azure import AzureNetwork


class AzureNetworking(Networking, Component):

    def get_cloud_driver(self):
        return self.driver.get_cloud_driver()

    def create_network(self, cidr, name=None, dns=False):

        print "Creando Red Virtual Azure ..."

        cloud_driver = self.get_cloud_driver()
        cloud_network = cloud_driver.ex_create_network(cidr_block=cidr,
                                                       name=name,
                                                       resource_group=self.driver.prop.resource_group,
                                                       location_id=self.driver.prop.location_id)

        network = AzureNetwork(cloud_network.id,
                               cloud_network.name,
                               cidr,
                               self)

        # Create the VPN Subnet
        network.create_vpn_subnet()

        # Create the DNS Subnet
        if dns:
            network.create_dns_subnet()

        print "Red Virtual Azure creada con nombre='{0}', cidr='{1}'.".format(
            network.name,
            cidr)

        return network

    def get_network(self, network_id):
        pass

    def list_networks(self):
        cloud_driver = self.get_cloud_driver()
        networks = cloud_driver.ex_list_networks()
        result = list()
        for n in networks:
            result.append(AzureNetwork(n.id,
                                       n.name,
                                       n.extra["addressSpace"]["addressPrefixes"],
                                       self))
        return result

    def create_gateway(self, name=None):
        raise NotImplementedError('create_gateway not implemented for this driver')



# Virtual Machine para Azure
import subprocess

from bastion.compute.vm.base import VirtualMachine


class AzureVirtualMachine(VirtualMachine):

    def get_cloud_driver(self):
        return self.compute.get_cloud_driver()

    def start(self):
        cloud_driver = self.get_cloud_driver()

        vm = cloud_driver.list_nodes(ex_node_ids=[self.id])[0]
        cloud_driver.ex_start_node(vm)

    def stop(self):
        cloud_driver = self.get_cloud_driver()

        vm = cloud_driver.list_nodes(ex_node_ids=[self.id])[0]
        cloud_driver.ex_stop_node(vm)

    def restart(self):
        cloud_driver = self.get_cloud_driver()

        vm = cloud_driver.list_nodes(ex_node_ids=[self.id])[0]
        cloud_driver.reboot_node(vm)

    def attach_nic(self, nic):

        cloud_driver = self.get_cloud_driver()
        cloud_driver.ex_attach_network_interface(self.name, nic.id, self.compute.driver.prop.resource_group, self.compute.driver.prop.location_id)

    def provision(self, playbook_path, additional_options=[], parameters=None, user='azureuser'):

        if parameters is None:
            cmd = ['ansible-playbook', '-i', self.public_ips[0] + ',', playbook_path,
                   '--private-key=' + self.compute.driver.cred.private_key_path,
                   '-u', user]
        else:
            cmd = ['ansible-playbook', '-i', self.public_ips[0] + ',', playbook_path,
                   '-e', parameters,
                   '--private-key=' + self.compute.driver.cred.private_key_path,
                   '-u', user]

        cmd.extend(additional_options)

        p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)

        for line in iter(p.stdout.readline, b''):
            print(">>> " + line.rstrip())

    def delete(self):
        cloud_driver = self.get_cloud_driver()

        vm = cloud_driver.list_nodes(ex_node_ids=[self.id])[0]
        cloud_driver.destroy_node(vm)

    def add_hostname(self, hostname, domain=None):
        private_dns = self.subnet.network.private_dns
        if domain is None and private_dns.domain is None:
            raise NotImplementedError(
                'domain is required')
        if domain is not None and private_dns.domain is not None and domain != private_dns.domain:
            raise NotImplementedError(
                'domain must be the same default domain of the dns created')
        private_dns.domain = domain
        private_dns.add_hostname(hostname, self)



from bastion.provider_types import Providers
from bastion.compute.aws import AWSBaseCompute
from bastion.networking.aws import AWSNetworking
from bastion.compute.azure import AzureBaseCompute
from bastion.networking.azure import AzureNetworking
from libcloud.compute.types import Provider
from bastion.libcloudbastion.driver import get_driver
from bastion.component import Component

from bastion.libcloudbastion.driver_dns import get_driver as get_dns_driver
from libcloud.dns.types import Provider as DNSProvider


class Driver:

    prop = None
    cred = None

    def __init__(self):
        pass

    @staticmethod
    def create(provider_info):

        if provider_info.type == Providers.AWS:
            driver = AWSDriver(provider_info)
        if provider_info.type == Providers.AZURE:
            driver = AzureDriver(provider_info)

        return driver


class AzureDriver(Driver, Component):

    def __init__(self, provider_info):
        cls = get_driver(Provider.AZURE_ARM)
        self._cloud_driver = cls(tenant_id=provider_info.cred.tenant_id, subscription_id=provider_info.cred.subscription_id, key=provider_info.cred.application_id, secret=provider_info.cred.password)
        self.baseCompute = AzureBaseCompute(self)
        self.networking = AzureNetworking(self)
        self.prop = provider_info.prop
        self.cred = provider_info.cred

    def get_cloud_driver(self):
        return self._cloud_driver


class AWSDriver(Driver, Component):

    def __init__(self, provider_info):
        cls = get_driver(Provider.EC2)
        self._cloud_driver = cls(provider_info.cred.access_key, provider_info.cred.secret_key, region=provider_info.cred.region)
        self.baseCompute = AWSBaseCompute(self)
        self.networking = AWSNetworking(self)
        self.prop = provider_info.prop
        self.cred = provider_info.cred
        cls = get_dns_driver(DNSProvider.ROUTE53)
        dns_credentials = (provider_info.cred.access_key, provider_info.cred.secret_key)
        self.dns_driver = cls(*dns_credentials)

    def get_cloud_driver(self):
        return self._cloud_driver




import time

from libcloud.compute.drivers.azure_arm import AzureNodeDriver as LibCloudAzureNodeDriver
from libcloud.compute.drivers.azure_arm import AzureNetwork, AzureSubnet, AzureNic
from libcloud.common.types import LibcloudError


class AzureLocalNetworkGateway:
    def __init__(self, id, name, address_space, gateway_ip_address):
        self.id = id
        self.name = name
        self.address_space = address_space
        self.gatewayIpAddress = gateway_ip_address


class AzureVirtualPrivateGateway:
    def __init__(self, id, name, public_ip):
        self.id = id
        self.name = name
        self.public_ip = public_ip


class AzureConnection:
    def __init__(self, id, name):
        self.id = id
        self.name = name


class AzureNodeDriver(LibCloudAzureNodeDriver):

    def ex_update_network(self, resource_group, location_id, cidr_block, name, dns_server):

        action = "/subscriptions/%s/resourceGroups/%s/providers/" \
                 "Microsoft.Network/virtualNetworks/%s" \
                 % (self.connection.subscription_id, resource_group, name)

        r = self.connection.request(action,
                                    params={"api-version": "2017-08-01"},
                                    method="GET")

        data = r.object

        data["properties"]["dhcpOptions"] = {
                                                "dnsServers": [
                                                    dns_server
                                                ]
                                            }

        r = self.connection.request(action,
                                    params={"api-version": "2017-08-01"},
                                    method="PUT",
                                    data=data)

        return AzureNetwork(r.object["id"],
                            r.object["name"],
                            r.object["location"],
                            r.object["properties"])

    def ex_create_network(self, resource_group, location_id, cidr_block, dns_server=None, name=None):

        data = {
            "location": location_id,
            "properties": {
                "addressSpace": {
                    "addressPrefixes": [
                        cidr_block
                    ]
                }
            }
        }

        action = "/subscriptions/%s/resourceGroups/%s/providers/" \
                 "Microsoft.Network/virtualNetworks/%s" \
                 % (self.connection.subscription_id, resource_group, name)

        r = self.connection.request(action,
                                    params={"api-version": "2017-03-01"},
                                    method="PUT",
                                    data=data)
        return AzureNetwork(r.object["id"],
                            r.object["name"],
                            r.object["location"],
                            r.object["properties"])

    def ex_delete_network(self, resource_group, name):

        action = "/subscriptions/%s/resourceGroups/%s/providers/" \
                 "Microsoft.Network/virtualNetworks/%s" \
                 % (self.connection.subscription_id, resource_group, name)

        r = self.connection.request(action,
                                    params={"api-version": "2017-03-01"},
                                    method="DELETE")

    def ex_create_subnet(self, resource_group, cidr_block, network_name, name=None):

        data = {
            "properties": {
                "addressPrefix": cidr_block
            }
        }

        action = "/subscriptions/%s/resourceGroups/%s/providers/" \
                 "Microsoft.Network/virtualNetworks/%s/subnets/%s" \
                 % (self.connection.subscription_id, resource_group, network_name, name)

        r = self.connection.request(action,
                                    params={"api-version": "2017-03-01"},
                                    method="PUT",
                                    data=data)

        return AzureSubnet(r.object["id"],
                           r.object["name"],
                           r.object["properties"])

    def ex_delete_subnet(self, resource_group, name, network_name):

        action = "/subscriptions/%s/resourceGroups/%s/providers/" \
                 "Microsoft.Network/virtualNetworks/%s/subnets/%s" \
                 % (self.connection.subscription_id, resource_group, network_name, name)

        r = self.connection.request(action,
                                    params={"api-version": "2016-09-01"},
                                    method="DELETE")

    def ex_create_public_ip(self, name, resource_group, location=None, allocation_method='Dynamic'):
        """
        Create a public IP resources.

        :param name: Name of the public IP resource
        :type name: ``str``

        :param resource_group: The resource group to create the public IP
        :type resource_group: ``str``

        :param location: The location at which to create the public IP
        (if None, use default location specified as 'region' in __init__)
        :type location: :class:`.NodeLocation`

        :param allocation_method:
        :type location: ``str``

        :return: The newly created public ip object
        :rtype: :class:`.AzureIPAddress`
        """

        if location is None:
            if self.default_location:
                location = self.default_location
            else:
                raise ValueError("location is required.")

        target = "/subscriptions/%s/resourceGroups/%s/" \
                 "providers/Microsoft.Network/publicIPAddresses/%s" \
                 % (self.subscription_id, resource_group, name)
        data = {
            "location": location.id,
            "tags": {},
            "properties": {
                "publicIPAllocationMethod": allocation_method
            }
        }
        r = self.connection.request(target,
                                    params={"api-version": "2017-09-01"},
                                    data=data,
                                    method='PUT'
                                    )
        return self._to_ip_address(r.object)

    # def ex_get_public_ip(self, name, resource_group):
    #     """
    #     Create a public IP resources.
    #
    #     :param name: Name of the public IP resource
    #     :type name: ``str``
    #
    #     :param resource_group: The resource group to create the public IP
    #     :type resource_group: ``str``
    #
    #     """
    #
    #     "/Microsoft.Network/publicIPAddresses/{publicIpAddressName}?api-version=2017-09-01"
    #
    #     target = "/subscriptions/%s/resourceGroups/%s/" \
    #              "providers/Microsoft.Network/publicIPAddresses/%s" \
    #              % (self.subscription_id, resource_group, name)
    #
    #     r = self.connection.request(target,
    #                                 params={"api-version": "2017-09-01"},
    #                                 method='GET'
    #                                 )
    #     return self._to_ip_address(r.object)

    def wait_until_public_ip_running(self, public_ip_id, wait_period=3, timeout=600):
        """
        Block until the provided nodes are considered running.

        Node is considered running when it's state is "running" and when it has
        at least one IP address assigned.

        :param public_ip_id: Public IP Id.
        :type public_ip_id: ``str``

        :param wait_period: How many seconds to wait between each loop
                            iteration. (default is 3)
        :type wait_period: ``int``

        :param timeout: How many seconds to wait before giving up.
                        (default is 600)
        :type timeout: ``int``
        """

        start = time.time()
        end = start + timeout

        while time.time() < end:

            pub_addr = self.ex_get_public_ip(public_ip_id)
            addr = pub_addr.extra.get("ipAddress")
            if addr:
                return pub_addr
            else:
                time.sleep(wait_period)
                continue

        raise LibcloudError(value='Timed out after %s seconds' % (timeout),
                            driver=self)

    def ex_create_network_interface(self, name, subnet, resource_group,
                                    location=None, public_ip=None, private_ip=None):
        """
        Create a virtual network interface (NIC).

        :param name: Name of the NIC resource
        :type name: ``str``

        :param subnet: The subnet to attach the NIC
        :type subnet: :class:`.AzureSubnet`

        :param resource_group: The resource group to create the NIC
        :type resource_group: ``str``

        :param location: The location at which to create the NIC
        (if None, use default location specified as 'region' in __init__)
        :type location: :class:`.NodeLocation`

        :param public_ip: Associate a public IP resource with this NIC
        (optional).
        :type public_ip: :class:`.AzureIPAddress`

        :return: The newly created NIC
        :rtype: :class:`.AzureNic`
        """

        if location is None:
            if self.default_location:
                location = self.default_location
            else:
                raise ValueError("location is required.")

        target = "/subscriptions/%s/resourceGroups/%s/providers" \
                 "/Microsoft.Network/networkInterfaces/%s" \
                 % (self.subscription_id, resource_group, name)

        data = {
            "location": location.id,
            "tags": {},
            "properties": {
                "ipConfigurations": [{
                    "name": "myip1",
                    "properties": {
                        "subnet": {
                            "id": subnet.id
                        },
                        "privateIPAllocationMethod": "Dynamic"
                    }
                }]
            }
        }

        if private_ip:
            data["properties"]["ipConfigurations"][0]["properties"]["privateIPAllocationMethod"] = "Static"
            data["properties"]["ipConfigurations"][0]["properties"]["privateIPAddress"] = private_ip

        if public_ip:
            data["properties"]["ipConfigurations"][0]["properties"]["publicIPAddress"] = {
                "id": public_ip.id
            }

        r = self.connection.request(target,
                                    params={"api-version": "2016-09-01"},
                                    data=data,
                                    method='PUT'
                                    )
        return AzureNic(r.object["id"], r.object["name"], r.object["location"],
                        r.object["properties"])

    def ex_list_network_interfaces(self, resource_group):

        action = "/subscriptions/%s/resourceGroups/%s/providers" \
                 "/Microsoft.Network/networkInterfaces" \
                 % (self.subscription_id, resource_group)
        r = self.connection.request(action,
                                    params={"api-version": "2017-09-01"},
                                    method="GET"
                                    )
        return [AzureNic(nic["id"], nic["name"], nic["location"],
                         nic["properties"]) for nic in r.object["value"]]

    def ex_create_local_network_gateway(self, name, address_space, gateway_ip_address, resource_group,
                                        location=None):

        if location is None:
            if self.default_location:
                location = self.default_location
            else:
                raise ValueError("location is required.")

        target = "/subscriptions/%s/resourceGroups/%s/providers" \
                 "/Microsoft.Network/localNetworkGateways/%s" \
                 % (self.subscription_id, resource_group, name)

        data = {
            "location": location.id,
            "tags": {},
            "properties": {
                "localNetworkAddressSpace": {
                    "addressPrefixes": [
                        address_space
                    ]
                },
                "gatewayIpAddress": gateway_ip_address,
            }
        }

        r = self.connection.request(target,
                                    params={"api-version": "2017-03-01"},
                                    data=data,
                                    method='PUT'
                                    )

        return AzureLocalNetworkGateway(r.object["id"],
                                        r.object["name"],
                                        address_space,
                                        gateway_ip_address)

    def ex_create_virtual_network_gateway(self, name, subnet, resource_group,
                                          location=None, public_ip=None):

        target = "/subscriptions/%s/resourceGroups/%s/providers" \
                 "/Microsoft.Network/virtualNetworkGateways/%s" \
                 % (self.subscription_id, resource_group, name)

        data = {
            "location": location.id,
            "tags": {},
            "properties": {
                "ipConfigurations": [{
                    "properties": {
                        "privateIPAllocationMethod": "Dynamic",
                        "subnet": {
                            "id": subnet.id
                        },
                        "publicIPAddress": {
                            "id": public_ip.id
                        }
                    },
                    "name": name
                }],
                "gatewayType": "Vpn",
                "vpnType": "RouteBased"
            }
        }

        r = self.connection.request(target,
                                    params={"api-version": "2017-03-01"},
                                    data=data,
                                    method='PUT'
                                    )

        return AzureVirtualPrivateGateway(r.object["id"],
                                          r.object["name"],
                                          public_ip)

    def ex_create_virtual_network_gateway_connection(self, name, resource_group, shared_key,
                                                     location=None, local_network_gateway=None,
                                                     virtual_network_gateway=None):

        target = "/subscriptions/%s/resourceGroups/%s/providers" \
                 "/Microsoft.Network/connections/%s" \
                 % (self.subscription_id, resource_group, name)

        data = {
            "location": location.id,
            "tags": {},
            "properties": {
                "sharedKey": shared_key,
                "virtualNetworkGateway1": {
                    "id": virtual_network_gateway.id
                },
                "localNetworkGateway2": {
                    "id": local_network_gateway.id
                },
                "connectionType": "Ipsec"
            }
        }

        r = self.connection.request(target,
                                    params={"api-version": "2017-03-01"},
                                    data=data,
                                    method='PUT'
                                    )

        return AzureConnection(r.object["id"],
                               r.object["name"])

    def ex_get_virtual_network_gateway_state(self, name, resource_group):

        target = "/subscriptions/%s/resourceGroups/%s/providers" \
                 "/Microsoft.Network/virtualNetworkGateways/%s" \
                 % (self.subscription_id, resource_group, name)

        r = self.connection.request(target,
                                    params={"api-version": "2017-03-01"},
                                    method='GET'
                                    )

        return r.object["properties"]["ipConfigurations"][0]["properties"]["provisioningState"]

    def ex_delete_network_interface(self, name, resource_group):

        target = "/subscriptions/%s/resourceGroups/%s/providers" \
                 "/Microsoft.Network/networkInterfaces/%s" \
                 % (self.subscription_id, resource_group, name)

        r = self.connection.request(target,
                                    params={"api-version": "2017-09-01"},
                                    method='DELETE'
                                    )

    def ex_attach_network_interface(self, vm_name, nic_id, resource_group, location_id):

        target = "/subscriptions/%s/resourceGroups/%s/providers" \
                 "/Microsoft.Compute/virtualMachines/%s" \
                 % (self.subscription_id, resource_group, vm_name)

        data = {
            "location": location_id,
            "properties": {
                "networkProfile": {
                    "networkInterfaces": [
                        {
                            "id": nic_id,
                            "properties": {
                                "primary": "true"
                            }
                        }
                    ]
                }
            }
        }

        r = self.connection.request(target,
                                    params={"api-version": "2017-12-01"},
                                    data=data,
                                    method='PUT'
                                    )





class PrivateDns:

    domain = None
    dns_server = None
    network = None

    def __init__(self, dns_server, network):
        self.dns_server = dns_server
        self.network = network

    def add_hostname(self, hostname, vm):
        raise NotImplementedError('add_hostname not implemented for this driver')





class Size:

    def __init__(self, id, name, ram, disk, bandwidth, price):
        self.id = id
        self.name = name
        self.ram = ram
        self.disk = disk
        self.bandwith = bandwidth
        self.price = price



# Security Group para AWS

from bastion.networking.security_group.base import SecurityGroup
from bastion.component import Component


class AWSSecurityGroup (SecurityGroup, Component):

    def get_cloud_driver(self):
        return self.network.get_cloud_driver()

    def allow_inbound_traffic(self, port):
        cloud_driver = self.get_cloud_driver()
        cloud_driver.ex_authorize_security_group_ingress(id=self.id,
                                                         from_port=port,
                                                         to_port=port,
                                                         cidr_ips=["0.0.0.0/0"])

    def delete(self):
        cloud_driver = self.get_cloud_driver()
        cloud_driver.ex_delete_security_group_by_id(group_id=self.id)



from libcloud.dns.drivers.route53 import Route53DNSDriver as LibCloudRoute53DNSDriver
from libcloud.dns.base import Zone

from libcloud.utils.xml import findall, findtext

import uuid

try:
    from lxml import etree as ET
except ImportError:
    from xml.etree import ElementTree as ET

API_VERSION = '2013-04-01'
API_HOST = 'route53.amazonaws.com'
API_ROOT = '/%s/' % (API_VERSION)

NAMESPACE = 'https://%s/doc%s' % (API_HOST, API_ROOT)


class Route53DNSDriver(LibCloudRoute53DNSDriver):

    def ex_create_private_zone(self, domain, vpc_id, region):
        zone = ET.Element('CreateHostedZoneRequest', {'xmlns': NAMESPACE})
        ET.SubElement(zone, 'Name').text = domain
        ET.SubElement(zone, 'CallerReference').text = str(uuid.uuid4())
        hzg = ET.SubElement(zone, 'HostedZoneConfig')
        ET.SubElement(hzg, 'PrivateZone').text = "true"
        vpc = ET.SubElement(zone, 'VPC')
        ET.SubElement(vpc, 'VPCId').text = vpc_id
        ET.SubElement(vpc, 'VPCRegion').text = region
        uri = API_ROOT + 'hostedzone'
        data = ET.tostring(zone)
        rsp = self.connection.request(uri, method='POST', data=data).object
        elem = findall(element=rsp, xpath='HostedZone', namespace=NAMESPACE)[0]
        return self._to_zone(elem=elem)

    def _to_zone(self, elem):
        name = findtext(element=elem, xpath='Name', namespace=NAMESPACE)
        id = findtext(element=elem, xpath='Id',
                      namespace=NAMESPACE).replace('/hostedzone/', '')
        comment = findtext(element=elem, xpath='Config/Comment',
                           namespace=NAMESPACE)
        resource_record_count = int(findtext(element=elem,
                                             xpath='ResourceRecordSetCount',
                                             namespace=NAMESPACE))

        extra = {'Comment': comment, 'ResourceRecordSetCount':
                 resource_record_count}

        zone = Zone(id=id, domain=name, type='master', ttl=0, driver=self,
                    extra=extra)
        return zone


# Networking para AWS

from bastion.component import Component
from bastion.networking.base import Networking
from bastion.networking.network.aws import AWSNetwork
from bastion.networking.gateway.aws import AWSGateway


class AWSNetworking (Networking, Component):

    def get_cloud_driver(self):
        return self.driver.get_cloud_driver()

    def create_network(self, cidr, name=None, dns=False):

        print "Creando Red Virtual EC2 ..."

        cloud_driver = self.get_cloud_driver()
        network = cloud_driver.ex_create_network(name=name, cidr_block=cidr)
        n = AWSNetwork(network.id,
                       network.name,
                       network.cidr_block,
                       self)

        # TODO: Improve this! (Provisioning)
        gateway_name = "%s-gateway" % name
        gateway = self.create_gateway(gateway_name)
        n.attach_gateway(gateway)
        route_table = n.list_route_tables()[0]
        route_table.create_route('0.0.0.0/0', gateway)
        security_group = n.list_security_groups()[0]
        security_group.allow_inbound_traffic(22)

        # Create the VPN Subnet
        n.create_vpn_subnet()

        # Create the DNS Subnet
        if dns:
            n.create_dns_subnet()

        # EnableDnsHostnames
        cloud_driver.ex_network_enable_dns_hostnames(n)

        print "Red Virtual EC2 creada con nombre='{0}', cidr='{1}'.".format(
            network.name,
            cidr)

        return n

    def get_network(self, network_id):
        cloud_driver = self.get_cloud_driver()
        networks = cloud_driver.ex_list_networks(network_ids=[network_id])

        if networks:
            return AWSNetwork(networks[0].id,
                              networks[0].name,
                              networks[0].cidr_block,
                              self)

        return None

    def list_networks(self):
        cloud_driver = self.get_cloud_driver()
        networks = cloud_driver.ex_list_networks()
        result = list()
        for n in networks:
            result.append(AWSNetwork(n.id,
                                     n.name,
                                     n.cidr_block,
                                     self))
        return result

    def create_gateway(self, name=None):
        cloud_driver = self.get_cloud_driver()
        cloud_gateway = cloud_driver.ex_create_internet_gateway(name)
        return AWSGateway(cloud_gateway.id, name, self)


import os
import hvac

from bastion.provider_types import Providers
from bastion.authentication.base import BaseAuthDriver, AWSProviderInfo, AzureProviderInfo


class VaultAuthDriver(BaseAuthDriver):
    _client = None

    def __init__(self, token):
        self._client = hvac.Client(url=os.environ['VAULT_ADDR'])
        self._client.token = token

    def get_provider_info(self, path):
        secret = self._client.read(path)
        provider_info = None

        if secret["data"]["cloud_provider"] == Providers.AWS:
            provider_info = AWSProviderInfo(secret["data"])
        if secret["data"]["cloud_provider"] == Providers.AZURE:
            provider_info = AzureProviderInfo(secret["data"])

        return provider_info

    def get_providers_info(self, path):
        providers_info = []

        secret_lists = self._client.list(path)

        # Iterar sobre la lista de secrets
        for key in secret_lists["data"]["keys"]:
            provider_info = self.get_provider_info(path + '/' + key)
            if provider_info is not None:
                providers_info.append(provider_info)

        return providers_info



# Clases base


class NetworkInterface:

    def __init__(self, id, name, private_ip, subnet):
        self.id = id
        self.name = name
        self.private_ip = private_ip
        self.subnet = subnet

    def delete(self):
            raise NotImplementedError('delete not implemented for this driver')


# Clases base


class VirtualMachine:

    def __init__(self, id, name, size, image, compute, primary_network_interface=None, subnet=None):
        self.id = id
        self.name = name
        self.size = size
        self.image = image
        self.compute = compute
        self.public_ips = []
        self.private_ips = []
        self.primary_network_interface = primary_network_interface
        self.subnet = subnet

    def start(self):
        raise NotImplementedError(
            'start not implemented for this driver')

    def stop(self):
        raise NotImplementedError(
            'stop not implemented for this driver')

    def restart(self):
        raise NotImplementedError(
            'restart not implemented for this driver')

    def attach_nic(self, nic):
        raise NotImplementedError (
            'attach_nic not implemented for this driver')

    def provision(self, playbook_path, additional_options=[], parameters=None, user=None):
        raise NotImplementedError(
            'provision not implemented for this driver')

    def delete(self):
        raise NotImplementedError(
            'delete not implemented for this driver')

    def add_hostname(self, hostname, domain=None):
        raise NotImplementedError(
            'add_hostname not implemented for this driver')



# Clases base

import hashlib


class Networking:
    def __init__(self, driver):
        self.driver = driver

    def create_network(self, **kwargs):
        raise NotImplementedError('create_network not implemented for this driver')

    def get_network(self, network_id):
        raise NotImplementedError('list_networks not implemented for this driver')

    def list_networks(self):
        raise NotImplementedError('list_networks not implemented for this driver')

    def create_gateway(self, name=None):
        raise NotImplementedError('create_gateway not implemented for this driver')

    @staticmethod
    def connect_networks(network1, network2):

        print "Preparando conexion VPN ..."

        network1.prepare_vpn_connection()
        network2.prepare_vpn_connection()

        print "Esperando asignacion de IPs Publicas ..."

        network1_vpn_public_ip = network1.get_vpn_connection_public_ip()
        network2_vpn_public_ip = network2.get_vpn_connection_public_ip()

        print "Conectando terminales VPN ..."

        shared_key = hashlib.sha512().hexdigest()
        network1.connect_to_vpn_network(network2.cidr, network2_vpn_public_ip, shared_key)
        network2.connect_to_vpn_network(network1.cidr, network1_vpn_public_ip, shared_key)

        network1.start_vpn_connection()
        network2.start_vpn_connection()

        print "Conexion VPN establecida!"




# Network para AWS
import ipaddress
import json
import os
from libcloud.compute.drivers.ec2 import VPCInternetGateway, EC2Network

from bastion.component import Component
from bastion.libcloudbastion.ec2 import EC2VPNGateway
from bastion.networking.network.base import Network
from bastion.networking.private_dns.aws import AWSPrivateDns
from bastion.networking.route_table.aws import AWSRouteTable
from bastion.networking.security_group.aws import AWSSecurityGroup
from bastion.networking.subnet.aws import AWSSubnet


class AWSNetwork(Network, Component):

    vpn_gateway_private_ip = None

    def get_cloud_driver(self):
        return self.networking.get_cloud_driver()

    def list_route_tables(self):
        cloud_driver = self.get_cloud_driver()
        all_route_tables = cloud_driver.ex_list_route_tables()
        vpc_route_tables = []

        for lib_route_table in all_route_tables:
            if lib_route_table.extra['vpc_id'] == self.id:
                route_table = AWSRouteTable(lib_route_table.id, lib_route_table.name, lib_route_table.routes,
                                            lib_route_table.subnet_associations,
                                            lib_route_table.propagating_gateway_ids,
                                            self, extra=lib_route_table.extra)
                vpc_route_tables.append(route_table)

        return vpc_route_tables

    def list_security_groups(self):
        cloud_driver = self.get_cloud_driver()
        all_security_groups = cloud_driver.ex_get_security_groups()
        vpc_security_groups = []

        for lib_security_group in all_security_groups:
            if lib_security_group.extra['vpc_id'] == self.id:
                vpc_security_groups.append(AWSSecurityGroup(lib_security_group.id,
                                                            lib_security_group.name,
                                                            self))
        return vpc_security_groups

    def create_private_dns(self, domain):
        # Create Private Zone
        dns_server = self.networking.driver.dns_driver.ex_create_private_zone(domain=domain,
                                                                              vpc_id=self.id,
                                                                              region=self.networking.driver.cred.region)
        self.private_dns = AWSPrivateDns(domain, dns_server, self)
        return self.private_dns

    def create_subnet(self, cidr, name=None):

        print "Creando Subred en Red Virtual EC2 {} ...".format(self.name)

        cloud_driver = self.get_cloud_driver()

        availability_zones = cloud_driver.ex_list_availability_zones()
        zone = [az for az in availability_zones][0]

        subnet = cloud_driver.ex_create_subnet(vpc_id=self.id,
                                               name=name,
                                               cidr_block=cidr,
                                               availability_zone=zone.name)

        print "Subred creada con nombre='{0}', cidr='{1}'.".format(
            subnet.name,
            cidr)

        return AWSSubnet(subnet.id,
                         subnet.name,
                         cidr,
                         self)

    def create_vpn_subnet(self):
        network_mask_length = ipaddress.ip_network(self.cidr).prefixlen
        subnet_ip = ipaddress.ip_network(self.cidr).network_address + (2 ** (28 - network_mask_length) - 1) * 2 ** (
            32 - 28)
        subnet = ipaddress.ip_network((subnet_ip, 28))
        vpn_subnet_name = "%s-vpn-subnet" % self.name
        self.vpn_subnet = self.create_subnet(str(subnet), vpn_subnet_name)

    def prepare_vpn_connection(self):
        # Create VPN Terminal VM
        self.vpn_gateway = self.networking.driver.baseCompute.create_vm(name=self.name + "-vpn",
                                                                        image_id=self.networking.driver.prop.vpn_image_id,
                                                                        subnet=self.vpn_subnet)
        self.vpn_gateway.set_source_dest_check(False)
        self.vpn_gateway_ip = self.vpn_gateway.public_ips[0]
        self.vpn_gateway_private_ip = self.vpn_gateway.private_ips[0]

    def get_vpn_connection_public_ip(self):
        return self.vpn_gateway_ip

    def connect_to_vpn_network(self, cidr, public_ip, shared_key):

        vpn_parameters =\
            {
                'local_ip': self.vpn_gateway_private_ip,
                'local_public_ip': self.vpn_gateway_ip,
                'local_subnet': self.cidr,
                'remote_public_ip': public_ip,
                'remote_subnet': cidr,
                'psk': shared_key
            }

        route_table = self.list_route_tables()[0]
        route_table.create_route(cidr, internet_gateway=None, network_interface=self.vpn_gateway.primary_network_interface)

        cloud_driver = self.get_cloud_driver()
        default_security_group = self.list_security_groups()[0]

        cloud_driver.ex_authorize_security_group_by_id(default_security_group.id, 0, 65535, public_ip + '/32',
                                                       protocol='-1')

        this_file_path = os.path.abspath(os.path.dirname(__file__))
        strongswan_playbook_path = os.path.join(this_file_path, "../../playbooks/torian-strongswan.yml")

        self.vpn_gateway.provision(playbook_path=strongswan_playbook_path,
                                   parameters=json.dumps(vpn_parameters),
                                   user='ubuntu')

    def start_vpn_connection(self):
        this_file_path = os.path.abspath(os.path.dirname(__file__))
        ipsec_restart_path = os.path.join(this_file_path, "../../playbooks/ipsec-restart.yml")

        self.vpn_gateway.provision(playbook_path=ipsec_restart_path,
                                   user='ubuntu')

    def create_dns_subnet(self):
        network_mask_length = ipaddress.ip_network(self.cidr).prefixlen
        subnet_ip = ipaddress.ip_network(self.cidr).network_address + (2 ** (28 - network_mask_length) - 2) * 2 ** (
            32 - 28)
        subnet = ipaddress.ip_network((subnet_ip, 28))
        dns_subnet_name = "%s-dns-subnet" % self.name
        self.dns_subnet = self.create_subnet(str(subnet), dns_subnet_name)

    def list_subnets(self):
        cloud_driver = self.get_cloud_driver()
        all_subnets = cloud_driver.ex_list_subnets()
        vpc_subnets = []

        for lib_subnet in all_subnets:
            if lib_subnet.extra['vpc_id'] == self.id:
                vpc_subnet = AWSSubnet(lib_subnet.id,
                                       lib_subnet.name,
                                       lib_subnet.extra['cidr_block'],
                                       self)
                vpc_subnets.append(vpc_subnet)

        return vpc_subnets

    def delete(self):
        cloud_driver = self.get_cloud_driver()
        networks = cloud_driver.ex_list_networks(network_ids=[self.id])
        network = [n for n in networks][0]
        cloud_driver.ex_delete_network(network)

    def attach_gateway(self, gateway):
        cloud_driver = self.get_cloud_driver()
        cloud_gateway = VPCInternetGateway(id=gateway.id, name=None, vpc_id=None, state=None, driver=None)
        cloud_network = EC2Network(id=self.id, name=None, cidr_block=None)
        cloud_driver.ex_attach_internet_gateway(gateway=cloud_gateway, network=cloud_network)

    def attach_vpn_gateway(self, vpn_gateway):
        cloud_driver = self.get_cloud_driver()
        cloud_vpn_gateway = EC2VPNGateway(id=vpn_gateway.id, name=None, state=None)
        cloud_network = EC2Network(id=self.id, name=None, cidr_block=None)
        cloud_driver.ex_attach_vpn_gateway(vpn_gateway=cloud_vpn_gateway, network=cloud_network)



# Subnet para Amazon

from bastion.networking.network_interface.azure import AzureNetworkInterface
from bastion.networking.subnet.base import Subnet
from bastion.component import Component


class AzureSubnet (Subnet, Component):

    def get_cloud_driver(self):
        return self.network.get_cloud_driver()

    def create_nic(self, private_ip, name=None):
        cloud_driver = self.get_cloud_driver()
        locations = cloud_driver.list_locations()
        location = [l for l in locations if l.name == self.network.networking.driver.prop.location_id][0]
        nic = cloud_driver.ex_create_network_interface(name=name,
                                                       subnet=self,
                                                       resource_group=self.network.networking.driver.prop.resource_group,
                                                       location=location,
                                                       private_ip=private_ip)
        return AzureNetworkInterface(nic.id,
                                     nic.name,
                                     private_ip,
                                     self)

    def list_nics(self):
        cloud_driver = self.get_cloud_driver()
        all_nics = cloud_driver.ex_list_network_interfaces(resource_group=self.network.networking.driver.prop.resource_group)
        subnet_nics = []
        for lib_nic in all_nics:
            ips = lib_nic.extra["ipConfigurations"]
            for ip in ips:
                if ip["properties"]["subnet"]["id"] == self.id:
                    private_ip = ip["properties"]["privateIPAddress"]
                    subnet_nic = AzureNetworkInterface(lib_nic.id,
                                                       lib_nic.name,
                                                       private_ip,
                                                       self)
                    subnet_nics.append(subnet_nic)
        return subnet_nics

    def delete(self):
        cloud_driver = self.get_cloud_driver()
        cloud_driver.ex_delete_subnet(resource_group=self.network.networking.driver.prop.resource_group,
                                      name=self.name,
                                      network_name=self.network.name)



# Clases base


class BaseCompute:

    def __init__(self, driver):
        self.driver = driver

    def create_vm(self, name=None, image_id=None, size_id=None, subnet=None):
        """
        :param name:   String with a name for this new node
        :type name:   ``str``

        :param size_id:   The size identifier of resources allocated to this node.
        :type size_id:   ``str``

        :param image_id:   OS image identifier to boot on node.
        :type image_id:   ``str``

        :param subnet:   The subnet for this new node.
        :type subnet:   ``Subnet``

        """
        raise NotImplementedError(
            'create_vm not implemented for this driver')

    def list_vms(self):
        raise NotImplementedError(
            'list_vms not implemented for this driver')

    def list_images(self):
        raise NotImplementedError(
            'list_images not implemented for this driver')

    def get_image(self, image_id):
        raise NotImplementedError(
            'get_image not implemented for this driver')

    def list_sizes(self):
        raise NotImplementedError(
            'list_sizes not implemented for this driver')

    def get_size(self, size_id):
        raise NotImplementedError(
            'get_size not implemented for this driver')



# NetworkInterface para AWS

from bastion.networking.network_interface.base import NetworkInterface
from bastion.component import Component


class AzureNetworkInterface (NetworkInterface, Component):

    def get_cloud_driver(self):
        return self.subnet.get_cloud_driver()

    def delete(self):
        cloud_driver = self.get_cloud_driver()
        cloud_driver.ex_delete_network_interface(resource_group=self.driver.prop.res.resource_group,
                                                 name=self.name)



# Compute Driver para Azure
import os

from bastion.component import Component
from bastion.compute.base import BaseCompute
from bastion.compute.common.size import Size
from bastion.compute.common.image import Image
from libcloud.compute.base import NodeAuthSSHKey
from bastion.compute.vm.azure import AzureVirtualMachine


class AzureBaseCompute(BaseCompute, Component):

    def get_cloud_driver(self):
        return self.driver.get_cloud_driver()

    def create_vm(self, name=None, image_id=None, size_id=None, subnet=None):

        print "Creando Maquina Virtual Azure ..."

        cloud_driver = self.get_cloud_driver()

        if image_id is None:
            image_id = self.driver.prop.image_id

        if size_id is None:
            size_id = self.driver.prop.size_id

        if name is None:
            name = 'bas7ion-node'

        if subnet is None:
            network_name = "%s-network" % name
            network_cidr = "10.0.0.0/16"
            subnet_name = "%s-subnet" % name
            subnet_cidr = "10.0.0.0/24"
            net = self.driver.networking.create_network(name=network_name,
                                                        cidr=network_cidr)
            subnet = net.create_subnet(name=subnet_name,
                                       cidr=subnet_cidr)

        locations = cloud_driver.list_locations()
        location = [l for l in locations if l.name == self.driver.prop.location_id][0]

        public_ip_name = "%s-public-ip" % name
        public_ip = cloud_driver.ex_create_public_ip(public_ip_name,
                                                     resource_group=self.driver.prop.resource_group,
                                                     location=location)

        nic_name = "%s-nic" % name
        nic = cloud_driver.ex_create_network_interface(name=nic_name,
                                                       subnet=subnet,
                                                       resource_group=self.driver.prop.resource_group,
                                                       location=location,
                                                       public_ip=public_ip)

        sizes = cloud_driver.list_sizes(location=location)
        size = [s for s in sizes if s.id == size_id][0]

        image = cloud_driver.get_image(image_id=image_id,
                                       location=location)

        img = Image(image.id,
                    image.name)

        sz = Size(size.id,
                  size.name,
                  size.ram,
                  size.disk,
                  size.bandwidth,
                  size.price)

        key_file_path = os.path.expanduser(self.driver.cred.public_key_path)

        with open(key_file_path, 'r') as fp:
            key_material = fp.read()

        auth = NodeAuthSSHKey(key_material)

        vm = cloud_driver.create_node(name=name,
                                      size=size,
                                      image=image,
                                      auth=auth,
                                      location=location,
                                      ex_resource_group=self.driver.prop.resource_group,
                                      ex_storage_account=self.driver.prop.storage_account,
                                      ex_nic=nic)

        nodes_ips_tuples = cloud_driver.wait_until_running([vm],
                                                           wait_period=5,
                                                           timeout=1000,
                                                           ssh_interface='public_ips',
                                                           force_ipv4=True,
                                                           ex_list_nodes_kwargs={"ex_resource_group": self.driver.prop.resource_group})

        node_ip_tuple = nodes_ips_tuples[0]

        azure_vm = AzureVirtualMachine(vm.id,
                                       vm.name,
                                       sz,
                                       img,
                                       self, subnet=subnet)
        azure_vm.public_ips.append(node_ip_tuple[1][0])
        azure_vm.private_ips.append(node_ip_tuple[0].private_ips[0])

        print "Maquina Virtual Azure creada con nombre='{0}', ip_privada='{1}', ip_publica='{2}'.".format(
            azure_vm.name,
            azure_vm.private_ips[0],
            azure_vm.public_ips[0])

        return azure_vm

    def list_vms(self):
        cloud_driver = self.get_cloud_driver()

        nodes = cloud_driver.list_nodes()
        result = list()
        for node in nodes:
            size = self.get_size(node.extra["properties"]["hardwareProfile"]["vmSize"])
            image_ref = node.extra["properties"]["storageProfile"]["imageReference"]
            publisher = image_ref["publisher"]
            offer = image_ref["offer"]
            sku = image_ref["sku"]
            version = image_ref["version"]
            image_id = publisher + ":" + offer + ":" + sku + ":" + version
            image = self.get_image(image_id)
            result.append(AzureVirtualMachine(node.id,
                                              node.name,
                                              size,
                                              image,
                                              self))
        return result

    def list_images(self):
        cloud_driver = self.get_cloud_driver()

        locations = cloud_driver.list_locations()
        location = [l for l in locations if l.name == self.driver.prop.location_id][0]

        images = cloud_driver.list_images(location=location)
        result = list()
        for image in images:
            result.append(Image(image.id,
                                image.name))
        return result

    def get_image(self, image_id):
        cloud_driver = self.get_cloud_driver()

        locations = cloud_driver.list_locations()
        location = [l for l in locations if l.name == self.driver.prop.location_id][0]

        image = cloud_driver.get_image(image_id=image_id,
                                       location=location)

        return Image(image.id,
                     image.name)

    def list_sizes(self):
        cloud_driver = self.get_cloud_driver()

        locations = cloud_driver.list_locations()
        location = [l for l in locations if l.name == self.driver.prop.location_id][0]

        sizes = cloud_driver.list_sizes(location=location)
        result = list()
        for size in sizes:
            result.append(Size(size.id,
                               size.name,
                               size.ram,
                               size.disk,
                               size.bandwidth,
                               size.price))
        return result

    def get_size(self, size_id):
        cloud_driver = self.get_cloud_driver()

        locations = cloud_driver.list_locations()
        location = [l for l in locations if l.name == self.driver.prop.location_id][0]

        sizes = cloud_driver.list_sizes(location=location)
        size = [s for s in sizes if s.id == size_id][0]

        return Size(size.id,
                    size.name,
                    size.ram,
                    size.disk,
                    size.bandwidth,
                    size.price)



from libcloud.dns.providers import Provider
from libcloud.dns.providers import DRIVERS
from libcloud.dns.types import OLD_CONSTANT_TO_NEW_MAPPING
from libcloud.common.providers import get_driver as _get_provider_driver
from libcloud.common.providers import set_driver as _set_provider_driver

del DRIVERS[Provider.ROUTE53]
DRIVERS[Provider.ROUTE53] = ('bastion.libcloudbastion.route53', 'Route53DNSDriver')


def get_driver(provider):
    deprecated_constants = OLD_CONSTANT_TO_NEW_MAPPING
    return _get_provider_driver(drivers=DRIVERS, provider=provider,
                                deprecated_constants=deprecated_constants)


def set_driver(provider, module, klass):
    return _set_provider_driver(drivers=DRIVERS, provider=provider,
                                module=module, klass=klass)


# Network para Azure

import json
import ipaddress

from bastion.networking.network.base import Network
from bastion.networking.subnet.azure import AzureSubnet
from bastion.networking.private_dns.azure import AzurePrivateDns
from bastion.component import Component
from libcloud.compute.drivers.azure_arm import AzureNetwork


class AzureNetwork(Network, Component):

    dns_subnet = None

    def get_cloud_driver(self):
        return self.networking.get_cloud_driver()

    def list_route_tables(self):
        raise NotImplementedError('list_route_tables not implemented for this driver')

    def list_security_groups(self):
        raise NotImplementedError ('list_security_groups not implemented for this driver')

    def create_private_dns(self):
        cloud_driver = self.get_cloud_driver()
        # Create Private DNS Server
        dns_server = self.networking.driver.baseCompute.create_vm(name=self.name+"-ns1",
                                                                  image_id=self.networking.driver.prop.dns_image_id,
                                                                  subnet=self.dns_subnet)

        cloud_driver.ex_update_network(cidr_block=self.cidr,
                                       name=self.name,
                                       resource_group=self.networking.driver.prop.resource_group,
                                       location_id=self.networking.driver.prop.location_id,
                                       dns_server=dns_server.private_ips[0])

        self.private_dns = AzurePrivateDns(dns_server, self)
        return self.private_dns

    def create_subnet(self, cidr, name=None):

        print "Creando Subred en Red Virtual Azure {} ...".format(self.name)

        cloud_driver = self.get_cloud_driver()

        subnet = cloud_driver.ex_create_subnet(resource_group=self.networking.driver.prop.resource_group,
                                               cidr_block=cidr,
                                               name=name,
                                               network_name=self.name)

        azure_subnet = AzureSubnet(subnet.id,
                                   subnet.name,
                                   cidr,
                                   self)

        print "Subred creada con nombre='{0}', cidr='{1}'.".format(
            subnet.name,
            cidr)

        return azure_subnet

    def create_dns_subnet(self):
        # TODO: Create a file with the harcoded data
        network_mask_length = ipaddress.ip_network(self.cidr).prefixlen
        subnet_ip = ipaddress.ip_network(self.cidr).network_address + (2**(28-network_mask_length) - 2) * 2**(32-28)
        subnet = ipaddress.ip_network((subnet_ip, 28))
        dns_subnet_name = "%s-dns-subnet" % self.name
        self.dns_subnet = self.create_subnet(str(subnet), dns_subnet_name)
        self.create_private_dns()

    def create_vpn_subnet(self):
        # TODO: Create a file with the harcoded data
        network_mask_length = ipaddress.ip_network(self.cidr).prefixlen
        subnet_ip = ipaddress.ip_network(self.cidr).network_address + (2**(28-network_mask_length) - 1) * 2**(32-28)
        subnet = ipaddress.ip_network((subnet_ip, 28))

        self.vpn_subnet = self.create_subnet(str(subnet), "GatewaySubnet")

    def get_vpn_public_ip(self):
        return self.vpn_gateway.public_ip

    def prepare_vpn_connection(self):
        cloud_driver = self.get_cloud_driver()

        locations = cloud_driver.list_locations()
        location = [l for l in locations if l.name == self.networking.driver.prop.location_id][0]

        vpn_vng_public_ip_name = "%s-vpn-vng-public-ip" % self.name
        self.vpn_gateway_ip = cloud_driver.ex_create_public_ip(name=vpn_vng_public_ip_name,
                                                               resource_group=self.networking.driver.prop.resource_group,
                                                               location=location)

        vpn_vng_name = "%s-vpn-vng" % self.name
        self.vpn_gateway\
            = cloud_driver.ex_create_virtual_network_gateway(name=vpn_vng_name, subnet=self.vpn_subnet,
                                                             resource_group=self.networking.driver.prop.resource_group,
                                                             location=location, public_ip=self.vpn_gateway_ip)

    def get_vpn_connection_public_ip(self):
        cloud_driver = self.get_cloud_driver()

        self.vpn_gateway_ip = cloud_driver.wait_until_public_ip_running(self.vpn_gateway_ip.id,
                                                                        wait_period=30, timeout=3600)

        return self.vpn_gateway_ip.extra.get("ipAddress")

    def connect_to_vpn_network(self, cidr, public_ip, shared_key):
        cloud_driver = self.get_cloud_driver()

        locations = cloud_driver.list_locations()
        location = [l for l in locations if l.name == self.networking.driver.prop.location_id][0]

        vpn_lng_name = "%s-vpn-lng" % self.name
        vpn_lng = cloud_driver.ex_create_local_network_gateway(name=vpn_lng_name, address_space=cidr,
                                                               gateway_ip_address=public_ip,
                                                               resource_group=self.networking.driver.prop.resource_group,
                                                               location=location)

        vpn_conn_name = "%s-vpn-conn" % self.name
        cloud_driver.ex_create_virtual_network_gateway_connection(name=vpn_conn_name, shared_key=shared_key,
                                                                  resource_group=self.networking.driver.prop.resource_group,
                                                                  location=location, local_network_gateway=vpn_lng,
                                                                  virtual_network_gateway=self.vpn_gateway)

    def start_vpn_connection(self):
        pass

    def list_subnets(self):
        result = []

        cloud_driver = self.get_cloud_driver()

        n = AzureNetwork(self.id,
                         None,
                         None,
                         None)

        subnets = cloud_driver.ex_list_subnets(network=n)

        for subnet in subnets:
            result.append(AzureSubnet(subnet.id,
                                      subnet.name,
                                      subnet.extra["addressPrefix"],
                                      self))

        return result

    def delete(self):
        cloud_driver = self.get_cloud_driver()
        cloud_driver.ex_delete_network(resource_group=self.driver.prop.res.resource_group,
                                       name=self.name)

    def attach_gateway(self, gateway):
        raise NotImplementedError('attach_gateway not implemented for this driver')

    def refresh_dns_entries(self):
        self.private_dns.refresh_entries()



# Compute Driver para AWS
import os

from libcloud.compute.base import NodeAuthSSHKey
from bastion.component import Component
from bastion.compute.base import BaseCompute
from bastion.compute.common.image import Image
from bastion.compute.common.size import Size
from bastion.compute.vm.aws import AWSVirtualMachine
from bastion.networking.network_interface.aws import AWSNetworkInterface


class AWSBaseCompute(BaseCompute, Component):
    def get_cloud_driver(self):
        return self.driver.get_cloud_driver()

    def create_vm(self, name=None, image_id=None, size_id=None, subnet=None):

        print "Creando Maquina Virtual EC2 ..."

        cloud_driver = self.get_cloud_driver()

        if image_id is None:
            image_id = self.driver.prop.image_id

        if size_id is None:
            size_id = self.driver.prop.size_id

        if name is None:
            name = 'bas7ion-node'

        if subnet is None:
            network_name = "%s-network" % name
            network_cidr = "10.0.0.0/16"
            subnet_name = "%s-subnet" % name
            subnet_cidr = "10.0.0.0/24"
            net = self.driver.networking.create_network(name=network_name,
                                                        cidr=network_cidr)
            subnet = net.create_subnet(name=subnet_name,
                                       cidr=subnet_cidr)

        image = cloud_driver.get_image(image_id)

        sizes = cloud_driver.list_sizes()
        size = [s for s in sizes if s.id == size_id][0]

        img = Image(image.id,
                    image.name)

        sz = Size(size.id,
                  size.name,
                  size.ram,
                  size.disk,
                  size.bandwidth,
                  size.price)

        key_file_path = os.path.expanduser(self.driver.cred.public_key_path)

        with open(key_file_path, 'r') as fp:
            key_material = fp.read()

        auth = NodeAuthSSHKey(key_material)

        vm = cloud_driver.create_node(name=name,
                                      image=image,
                                      size=size,
                                      ex_subnet=subnet,
                                      ex_assign_public_ip=True,
                                      auth=auth)

        nodes_ips_tuples = cloud_driver.wait_until_running([vm],
                                                           wait_period=4,
                                                           timeout=600,
                                                           ssh_interface='public_ips',
                                                           force_ipv4=True)

        node_ip_tuple = nodes_ips_tuples[0]

        primary_network_interface = vm.extra['network_interfaces'][0]
        private_ip = primary_network_interface.extra['private_ips'][0][
            'private_ip']
        primary_network_interface = AWSNetworkInterface(id=primary_network_interface.id,
                                                        name=primary_network_interface.name,
                                                        private_ip=private_ip, subnet=None)
        aws_vm = AWSVirtualMachine(vm.id, vm.name, sz, img, self, primary_network_interface, subnet)
        aws_vm.public_ips.append(node_ip_tuple[1][0])
        aws_vm.private_ips.append(private_ip)

        print "Maquina Virtual EC2 creada con nombre='{0}', ip_privada='{1}', ip_publica='{2}'.".format(
            aws_vm.name,
            aws_vm.private_ips[0],
            aws_vm.public_ips[0])

        return aws_vm

    def list_vms(self):
        cloud_driver = self.get_cloud_driver()

        nodes = cloud_driver.list_nodes()
        result = list()
        for node in nodes:
            primary_network_interface = node.extra['network_interfaces'][0]
            private_ip = primary_network_interface.extra['private_ips'][0]['private_ip']
            primary_network_interface = AWSNetworkInterface(id=primary_network_interface.id,
                                                            name=primary_network_interface.name,
                                                            private_ip=private_ip, subnet=None)
            image = self.get_image(node.extra["image_id"])
            size = self.get_size(node.extra["instance_type"])
            result.append(AWSVirtualMachine(node.id,
                                            node.name,
                                            size,
                                            image,
                                            self,
                                            primary_network_interface))
        return result

    def list_images(self):
        cloud_driver = self.get_cloud_driver()

        images = cloud_driver.list_images()
        result = list()
        for image in images:
            result.append(Image(image.id,
                                image.name))
        return result

    def get_image(self, image_id):
        cloud_driver = self.get_cloud_driver()

        image = cloud_driver.list_images(ex_image_ids=[image_id])[0]
        return Image(image.id,
                     image.name)

    def list_sizes(self):
        cloud_driver = self.get_cloud_driver()

        sizes = cloud_driver.list_sizes()
        result = list()
        for size in sizes:
            result.append(Size(size.id,
                               size.name,
                               size.ram,
                               size.disk,
                               size.bandwidth,
                               size.price))
        return result

    def get_size(self, size_id):
        cloud_driver = self.get_cloud_driver()

        sizes = cloud_driver.list_sizes()
        size = [s for s in sizes if s.id == size_id][0]
        return Size(size.id,
                    size.name,
                    size.ram,
                    size.disk,
                    size.bandwidth,
                    size.price)



# Virtual Machine para AWS
import subprocess

from libcloud.compute.drivers.ec2 import EC2NetworkInterface
from bastion.compute.vm.base import VirtualMachine
from libcloud.compute.base import Node
from bastion.component import Component


class AWSVirtualMachine(VirtualMachine, Component):

    def get_cloud_driver(self):
        return self.compute.get_cloud_driver()

    def start(self):
        cloud_driver = self.get_cloud_driver()

        vm = cloud_driver.list_nodes(ex_node_ids=[self.id])[0]
        cloud_driver.ex_start_node(vm)

    def stop(self):
        cloud_driver = self.get_cloud_driver()

        vm = cloud_driver.list_nodes(ex_node_ids=[self.id])[0]
        cloud_driver.ex_stop_node(vm)

    def restart(self):
        cloud_driver = self.get_cloud_driver()

        vm = cloud_driver.list_nodes(ex_node_ids=[self.id])[0]
        cloud_driver.reboot_node(vm)

    def attach_nic(self, nic):
        cloud_driver = self.get_cloud_driver()

        i = EC2NetworkInterface(nic.id,
                                nic.name,
                                None)

        n = Node(self.id,
                 self.name,
                 None,
                 [],
                 [],
                 None)

        nodes = cloud_driver.list_nodes(ex_node_ids=[self.id])
        cloud_driver.wait_until_running(nodes=nodes)

        device_index = 0
        nics = nodes[0].extra["network_interfaces"]
        index_list = list()
        for n in nics:
            if n.extra["attachment"]:
                index_list.append(n.extra["attachment"]["device_index"])
        if len(index_list) > 0:
            device_index = max(index_list) + 1

        attachment_id = cloud_driver.ex_attach_network_interface_to_node(network_interface=i,
                                                                         node=self,
                                                                         device_index=device_index)

        print attachment_id

    def provision(self, playbook_path, additional_options=[], parameters=None, user='ec2-user'):

        if parameters is None:
            cmd = ['ansible-playbook', '-i', self.public_ips[0] + ',', playbook_path,
                   '--private-key=' + self.compute.driver.cred.private_key_path,
                   '-u', user]
        else:
            cmd = ['ansible-playbook', '-i', self.public_ips[0] + ',', playbook_path,
                   '-e', parameters,
                   '--private-key=' + self.compute.driver.cred.private_key_path,
                   '-u', user]

        cmd.extend(additional_options)

        p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)

        for line in iter(p.stdout.readline, b''):
            print(">>> " + line.rstrip())

    def delete(self):
        cloud_driver = self.get_cloud_driver()

        vm = cloud_driver.list_nodes(ex_node_ids=[self.id])[0]
        return cloud_driver.destroy_node(vm)

    def set_source_dest_check(self, enabled):
        cloud_driver = self.get_cloud_driver()

        n = Node(self.id,
                 self.name,
                 None,
                 [],
                 [],
                 None)
        cloud_driver.ex_modify_instance_attribute(n, {'SourceDestCheck.Value': enabled})

    def add_hostname(self, hostname, domain=None):
        private_dns = self.subnet.network.private_dns
        if private_dns is None:
            if domain is None:
                raise NotImplementedError(
                    'domain is required')
            private_dns = self.subnet.network.create_private_dns(domain)
        if domain is not None and domain != private_dns.domain:
            raise NotImplementedError(
                'domain must be the same default domain of the dns created')
        private_dns.add_hostname(hostname, self)



# Clases base


class RouteTable:

    def __init__(self, id, name, routes, subnet_associations,
                 propagating_gateway_ids, network, extra=None):
        self.id = id
        self.name = name
        self.routes = routes
        self.subnet_associations = subnet_associations
        self.propagating_gateway_ids = propagating_gateway_ids
        self.network = network
        self.extra = extra or {}

    def list_routes(self):
            raise NotImplementedError('list_routes not implemented for this driver')

    def create_route(self, cidr, internet_gateway=None):
            raise NotImplementedError('create_route not implemented for this driver')

    def list_subnet_associations(self):
            raise NotImplementedError('list_subnet_associations not implemented for this driver')

    def create_subnet_association(self, subnet):
            raise NotImplementedError('create_subnet_associations not implemented for this driver')

    def delete(self):
            raise NotImplementedError('delete not implemented for this driver')



from bastion.networking.route_table.base import RouteTable


class AWSRouteTable(RouteTable):
    def get_cloud_driver(self):
        return self.network.get_cloud_driver()

    def list_routes(self):
        return self.routes

    def create_route(self, cidr, internet_gateway=None, network_interface=None):
        cloud_driver = self.get_cloud_driver()
        route = cloud_driver.ex_create_route(self, cidr, internet_gateway=internet_gateway,
                                             network_interface=network_interface)

        return route

    def list_subnet_associations(self):
        return self.subnet_associations

    def create_subnet_association(self, subnet):
        cloud_driver = self.get_cloud_driver()
        cloud_driver.ex_associate_route_table(self, subnet)

    def delete(self):
        raise NotImplementedError('delete not implemented for this driver')



# Clases base


class Subnet:

    def __init__(self, id, name, cidr, network):
        self.id = id
        self.name = name
        self.cidr = unicode(cidr)
        self.network = network

    def create_nic(self, private_ip, name=None):
        raise NotImplementedError('create_nic not implemented for this driver')

    def list_nics(self):
            raise NotImplementedError('list_nics not implemented for this driver')

    def delete(self):
            raise NotImplementedError('delete not implemented for this driver')


# Clases base


class Network:

    private_dns = None

    vpn_subnet = None
    vpn_gateway = None
    # TODO Crear componente VPN Gateway
    vpn_gateway_ip = None


    def __init__(self, id, name, cidr, networking):
        self.id = id
        self.name = name
        self.cidr = unicode(cidr)
        self.networking = networking

    def list_route_tables(self):
            raise NotImplementedError('list_route_tables not implemented for this driver')

    def list_security_groups(self):
        raise NotImplementedError('list_security_groups not implemented for this driver')

    def create_private_dns(self, domain):
        raise NotImplementedError('create_private_dns not implemented for this driver')

    def create_subnet(self, cidr, name=None):
            raise NotImplementedError('create_subnet not implemented for this driver')

    def list_subnets(self):
            raise NotImplementedError('list_subnets not implemented for this driver')

    def delete(self):
            raise NotImplementedError('delete not implemented for this driver')

    def attach_gateway(self, gateway):
            raise NotImplementedError('attach_gateway not implemented for this driver')

    def prepare_vpn_connection(self):
        raise NotImplementedError('prepare_vpn_connection not implemented for this driver')

    def get_vpn_connection_public_ip(self):
        raise NotImplementedError('get_vpn_connection_public_ip not implemented for this driver')

    def connect_to_vpn_network(self, cidr, public_ip, shared_key):
        raise NotImplementedError('connect_to_network not implemented for this driver')

    def start_vpn_connection(self):
        raise NotImplementedError('start_vpn_connection not implemented for this driver')

    # def refresh_dns_entries(self):
    #         raise NotImplementedError('refresh not implemented for this driver')




from bastion.networking.private_dns.base import PrivateDns
from bastion.component import Component
from libcloud.dns.base import RecordType


class AWSPrivateDns(PrivateDns, Component):

    def __init__(self, domain, dns_server, network):
        super(AWSPrivateDns, self).__init__(dns_server, network)
        self.domain = domain

    def get_cloud_driver(self):
        return self.network.get_cloud_driver()

    def add_hostname(self, hostname, vm):
        self.network.networking.driver.dns_driver.create_record(name=hostname,
                                                                zone=self.dns_server,
                                                                type=RecordType.A,
                                                                data=vm.private_ips[0])



# Subnet para AWS

from bastion.networking.network_interface.aws import AWSNetworkInterface
from bastion.networking.subnet.base import Subnet
from bastion.component import Component


class AWSSubnet (Subnet, Component):

    def get_cloud_driver(self):
        return self.network.get_cloud_driver()

    def create_nic(self, private_ip, name=None):

        cloud_driver = self.get_cloud_driver()

        subnets = cloud_driver.ex_list_subnets(subnet_ids=[self.id])
        subnet = [s for s in subnets][0]
        nic = cloud_driver.ex_create_network_interface(subnet,
                                                       name=name,
                                                       private_ip_address=private_ip)
        return AWSNetworkInterface(nic.id,
                                   nic.name,
                                   private_ip,
                                   self)

    def list_nics(self):
        cloud_driver = self.get_cloud_driver()
        all_nics = cloud_driver.ex_list_network_interfaces()
        subnet_nics = []
        for lib_nic in all_nics:
            if lib_nic.extra['subnet_id'] == self.id:
                private_ip = lib_nic.extra['private_ips'][0]['private_ip']
                subnet_nic = AWSNetworkInterface(lib_nic.id,
                                                 lib_nic.name,
                                                 private_ip,
                                                 self)
                subnet_nics.append(subnet_nic)
        return subnet_nics

    def delete(self):
        cloud_driver = self.get_cloud_driver ()
        subnets = cloud_driver.ex_list_subnets(subnet_ids=[self.id])
        subnet = [s for s in subnets][0]
        cloud_driver.ex_delete_subnet(subnet)